@@ -7,6 +7,27 @@
 
 # source("d:/mwp/eanalysis/ff/R/zzz.R")
 
+# require(rhub)
+# rhub_ff_4.0.3 <- check_for_cran(
+#   path = "../ff_4.0.3.tar.gz"
+# , email = "Jens.Oehlschlaegel@truecluster.com"
+# , check_args = "--as-cran"
+# , env_vars = c('_R_CHECK_FORCE_SUGGESTS_'= "false",'_R_CHECK_CRAN_INCOMING_USE_ASPELL_'= "true", '_R_CHECK_XREFS_MIND_SUSPECT_ANCHORS_'="true")
+# , platform = c('macos-highsierra-release-cran','macos-highsierra-release')
+# , show_status = FALSE
+# )
+
+# > require(rhub)
+# > rhub_setup()
+# > rhub_doctor()
+# > rhub_check(platforms = c("linux", "macos", "windows", "ubuntu-clang", "ubuntu-gcc12"))
+# ✔ Found git repository at /home/jo/SIK/truecluster/ff.
+# ✔ Found GitHub PAT.                                 
+# ✔ Check started: linux, macos, windows, ubuntu-clang, ubuntu-gcc12 (aspherical-sphinx).
+# See <https://github.com/truecluster/ff/actions> for live output!
+
+
+
 .onLoad <- function(lib, pkg) {
   ##library.dynam("ff", pkg, lib) use useDynLib(ff, .registration = TRUE, .fixes = "C_") in NAMESPACE instead
   ##packageStartupMessage("Loading package ff", packageDescription("ff", lib, fields="Version"), "")
@@ -36,30 +57,12 @@
   if (is.null(getOption("ffdrop")))
     options(ffdrop=TRUE)
   if (is.null(getOption("ffbatchbytes"))){
-    # memory.limit is windows specific
-    if (.Platform$OS.type=="windows")
-    {
-      if (getRversion()>="2.6.0")  # memory.limit was silently changed from 2.6.0 to return in MB instead of bytes
-        options(ffbatchbytes=memory.limit()*(1024^2/100))
-      else
-        options(ffbatchbytes=memory.limit()/100)
-    } else {
-      # some magic constant
-      options(ffbatchbytes=16*1024^2)
-    }
+    # some magic constant
+    options(ffbatchbytes=16*1024^2)
   }
   if (is.null(getOption("ffmaxbytes"))){
-    # memory.limit is windows specific
-    if (.Platform$OS.type=="windows")
-    {
-      if (getRversion()>="2.6.0")  # memory.limit was silently changed from 2.6.0 to return in MB instead of bytes
-        options(ffmaxbytes=0.5*memory.limit()*(1024^2))
-      else
-        options(ffmaxbytes=0.5*memory.limit())
-    } else {
-      # some magic constant
-      options(ffmaxbytes=0.5*1024^3)
-    }
+    # some magic constant
+    options(ffmaxbytes=0.5*1024^3)
   }
   # if we want an explicit list of ff objects, we should store them in an environment with hash=TRUE (much faster than a list)
   #assign(".fftemp", new.env(hash=TRUE), envir=globalenv())
@@ -176,7 +179,7 @@
   # }
 }
 
-.Last.lib <- function(libpath) {
+.onDetach <- function(libpath) {
    packageStartupMessage("Detaching package ff")
   # if (getRversion()<="2.10.0"){
     # packageStartupMessage('restoring [.AsIs')
